package UI_Converter;

import java.io.*;
import java.util.*;
import javax.xml.parsers.*;
import org.xml.sax.*;
import org.xml.sax.helpers.*;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;
import UI_Converter.util.FileManager;
import UI_Converter.adminrequest.*;
import UI_Converter.adminresponse.*;
import UI_Converter.bytecode.*;
import UI_Converter.common.*;
import UI_Converter.securedmsg.*;
import UI_Converter.transport.*;
import UI_Converter.uimemsg.*;

public class XMLmain extends DefaultHandler{

	// Main function : reads the input file and gives it to the sun's parser.
	static public void main(String[] args) {

		String TransportKeyFileName	= null;
		String InputFileName		= null;
		String OutputFileName		= null;

		// Parse arguments
		for (int i = 0; i < args.length; i++) {

			switch(i){
				case 0:
					TransportKeyFileName = args[i];
					FileManager.SetTransportKeyFile(TransportKeyFileName);

				case 1:
					InputFileName = args[i];
					FileManager.SetInputFileName(InputFileName);

				case 2:
					OutputFileName = args[i];
					FileManager.SetOutputFile(OutputFileName);
			}
		}

		if ( (TransportKeyFileName == null) || (InputFileName == null)  || (OutputFileName == null)) {
			usage();
		}


        // Create a JAXP SAXParserFactory and configure it
        SAXParserFactory spf = SAXParserFactory.newInstance();
		// It is a validating parser
        spf.setValidating(true);

        XMLReader xmlReader = null;
        try {
            // Create a JAXP SAXParser
            SAXParser saxParser = spf.newSAXParser();

            // Get the encapsulated SAX XMLReader
            xmlReader = saxParser.getXMLReader();
        } catch (Exception ex) {
            System.err.println(ex);
            System.exit(1);
        }

        // Set the ContentHandler of the XMLReader
        xmlReader.setContentHandler(new XMLmain());

        // Set an ErrorHandler before parsing
        //xmlReader.setErrorHandler(new MyErrorHandler(System.err));

        try {
            // Tell the XMLReader to parse the XML document
            xmlReader.parse(convertToFileURL(InputFileName));
        } catch (SAXException se) {
            System.err.println(se.getMessage());
            System.exit(1);
        } catch (IOException ioe) {
            System.err.println(ioe);
            System.exit(1);
        }
    }


    private static void usage() {
        System.err.println("Usage: XMLtoUSAT.XMLmain <TransportKeyFile> <InputFile> Optional<OutputFile>");
        System.exit(1);
    }


	// Convert from a filename to a file URL.
    private static String convertToFileURL(String filename) {
        // On JDK 1.2 and later, simplify this to:
        // "path = file.toURL().toString()".
        String path = new File(filename).getAbsolutePath();
        if (File.separatorChar != '/') {
            path = path.replace(File.separatorChar, '/');
        }
        if (!path.startsWith("/")) {
            path = "/" + path;
        }
        return "file:" + path;
    }

   private Element currentElement;

   private Stack parentStack = new Stack();

   private StringBuffer buffer = new StringBuffer();

   private Uitestml rootElement;

   private int indentLevel = 0;

   public static boolean PageElementJustUnderRootElement = true;

   // Parser calls this for each element in a document
   public void startElement(String namespaceURI, String localName, String qName, Attributes atts)
	   throws SAXException{

	    indentLevel++;
		StringBuffer IndentString = new StringBuffer();
		for (int i=1; i<= indentLevel; i++)
			IndentString.append("   ");

		System.out.println(IndentString + "parsing the tag "+qName);
		Element parent = currentElement;
		parentStack.push(parent);

		// Here is the element list as described in document
		// USAT Interpreter Test Language Specification V0.0.4

		if (qName.equals("ui-test-ml"))
		{
			rootElement = new Uitestml();
			currentElement = rootElement;
		}
		else {
			//************************************
			// ui-app-msg element and sub-elements
			//************************************
			if (qName.equals("ui-app-msg"))
			{
				currentElement = new Uiappmsg(atts);
			}
			else
			if (qName.equals("ui-me-msg"))
			{
				currentElement = new UiMeMsg();
			}
			else
			if (qName.equals("uio-msg-pull"))
			{
				currentElement = new UioMsgPull();
			}
			else
			if (qName.equals("uio-msg-adm"))
			{
				currentElement = new UioMsgAdmin();
			}
			else			 
			if (qName.equals("gwo-msg-pull"))
			{
				PageElementJustUnderRootElement = false;
				currentElement = new GwoMsgPull();
			}
			else
			if (qName.equals("gwo-msg-push"))
			{
				currentElement = new GwoMsgPush();
			}
			else
			if (qName.equals("gwo-msg-adm"))
			{
				currentElement = new GwoMsgAdmin();
			}
			else
			if (qName.equals("submit"))
			{
				currentElement = new Submit();	
			}
			else
			if (qName.equals("request-id"))
			{
				currentElement = new RequestId();	
			}
			else
			if (qName.equals("additional-info"))
			{
				currentElement = new AdditionalInfo();	
			}
			else
			if (qName.equals("ui-version"))
			{
				currentElement = new UIVersion();
			}
			else
			if (qName.equals("ui-profile"))
			{
				currentElement = new UIProfile();
			}
			else
			if (qName.equals("hash-url"))
			{
				currentElement = new HashURL();
			}
			else
			if (qName.equals("reception-buffer-size"))
			{
				currentElement = new ReceptionBufferSize();
			}
			else
			if (qName.equals("transmission-buffer-size"))
			{
				currentElement = new TransmissionBufferSize();
			}
			else
			if (qName.equals("page"))
			{
				currentElement = new USATml(atts);	
			}
			else		
			if (qName.equals("page-unlock-code"))
			{
				currentElement = new PageUnlockCode(atts);
			}
			else
			if (qName.equals("one-time-password"))
			{
				currentElement = new OneTimePassword();
			}
			else
			if (qName.equals("keep-alive-list"))
			{
				currentElement = new KeepAliveList();
			}
			else
			if (qName.equals("service-id"))
			{
				currentElement = new ServiceId();
			}
			else
			if (qName.equals("string-pool"))
			{
				currentElement = new StringPool();
			}
			else
			if (qName.equals("trh-modifier"))
			{
				currentElement = new TrhModifier(atts);
			}
			else			
			if (qName.equals("action"))
			{
				currentElement = new Action(atts);
			}
			else			
			if (qName.equals("nu"))
			{
				currentElement = new NavigationUnit(atts);
			}
			else
			if (qName.equals("anchor"))
			{
				currentElement = new Anchor();
			}
			else				
			if (qName.equals("ui-set-variable"))
			{
				currentElement = new SetVariable();
			}
			else				
			if (qName.equals("ui-assign-and-branch"))
			{
				currentElement = new AssignAndBranch();
			}	
			else				
			if (qName.equals("ui-extract"))
			{
				currentElement = new Extract(atts);
			}
			else			
			if(qName.equals("ui-go-back"))
			{
				currentElement = new GoBack(atts);
			}
			else			
			if (qName.equals("ui-branch-on-variable-value"))
			{
				currentElement = new BranchOnVariableValue();
			}
			else			
			if (qName.equals("ui-exit"))
			{
				currentElement = new Exit(atts);
			}
			else			
			if (qName.equals("ui-execute-usat-cmd"))
			{
				currentElement = new ExecuteUSATCommand(atts);
			}
			else
			if (qName.equals("tr-result-variable-id"))
			{
				currentElement = new GeneralResultVariable();
			}
			else				
			if (qName.equals("usat-cmd"))
			{
				currentElement = new USATCommandTLV();
			}
		 	else
			if (qName.equals("simple-tlv"))
			{
				currentElement = new SimpleTLV();
			}
		 	else
			if (qName.equals("simple-tlv-indicator"))
			{
				currentElement = new SimpleTLVIndicator(atts);
			}
		 	else
			if (qName.equals("ui-execute-nc"))
			{
				currentElement = new ExecuteNativeCommand(atts);
			}
			else			 
			if (qName.equals("nci"))
			{
				currentElement = new NCI();
			}
			else
			if (qName.equals("input-list"))
			{
				currentElement = new InputList();
			}
			else
			if (qName.equals("ui-get-length"))
			{
				currentElement = new GetLength();
			}
			else
			if (qName.equals("ui-get-tlv-value"))
			{
				currentElement = new GetTLVValue(atts);
			}
			else				
			if (qName.equals("ui-display-text"))
			{
				currentElement = new Display(atts);
			}
			else
			if (qName.equals("ui-get-input"))
			{
				currentElement = new Input(atts);
			}
			else				
			if (qName.equals("inline-value2"))
			{
				currentElement = new InlineValue2(atts);
			}
			else
			if (qName.equals("adm-request"))
			{
				currentElement = new Uiadreqml(atts);
			}
			if (qName.equals("install-page"))
			{
				currentElement = new InstallPage(atts);
			}
			else
			if (qName.equals("remove-page"))
			{
				currentElement = new RemovePage(atts);
			}
			else
			if (qName.equals("entering-menu-title"))
			{
				currentElement = new ConfigureMenuTitle();
			}
			else
			if (qName.equals("icon-identifier"))
			{
				currentElement = new IconId(atts);
			}
			else
			if (qName.equals("page-menu-text"))
			{
				currentElement = new ConfigureMenuTextToPage();
			}
			else
			if (qName.equals("remove-page-menu-text"))
			{
				currentElement = new RemoveMenuTextToPage();
			}
			else
			if (qName.equals("install-event"))
			{
				currentElement = new InstallEvent();
			}
			else
			if (qName.equals("remove-event"))
			{
				currentElement = new RemoveEvent();
			}
			else
			if (qName.equals("env-variables"))
			{
				currentElement = new ModifyEnvVar();
			}
			else
			if (qName.equals("wait-state-message"))
			{
				currentElement = new ModifyWaitStateMsg();
			}
			else
			if (qName.equals("config-data"))
			{
				currentElement = new ModifyConfigData(atts);
			}
			else
			if (qName.equals("bearer-data"))
			{
				currentElement = new ModifyBearerData();
			}
			else
			if (qName.equals("data-set-index"))
			{
				currentElement = new DataSetIndex();
			}
			else
			if (qName.equals("bearer-type"))
			{
				currentElement = new BearerType();
			}
			else
			if (qName.equals("install-plugin"))
			{
				currentElement = new InstallPlugIn();
			}
			else
			if (qName.equals("remove-plugin"))
			{
				currentElement = new RemovePlugIn();
			}
			else
			if (qName.equals("plugin-installation-data"))
			{
				currentElement = new PlugInInstallationData();
			}
			else
			if (qName.equals("parameter-list"))
			{
				currentElement = new ParameterList(atts);
			}
			else
			if (qName.equals("parameter"))
			{
				currentElement = new Parameter(atts);
			}
			else
			if (qName.equals("tar"))
			{
				currentElement = new Tar();
			}
			else
			if (qName.equals("event-identifier"))
			{
				currentElement = new EventIdentifier();
			}
			else
			if (qName.equals("adm-response"))
			{
				currentElement = new Uiadrespml();	
			}
			else
			if (qName.equals("adm-result"))
			{
				currentElement = new AdminResult(atts);
			}
			if (qName.equals("page-identification"))
			{
				currentElement = new PageIdentification();
			}
			else
			if (qName.equals("inline-value"))
			{
				currentElement = new InlineValue(atts);
			}
			else
			if (qName.equals("value"))
			{
				currentElement = new Value(atts);
			}
			else
			if (qName.equals("reference"))
			{
				currentElement = new Reference(atts);
			}
			else
			if (qName.equals("variable-identifier-list"))
			{
				currentElement = new VariableIdentifierList();
			}
			else
			if (qName.equals("destination-variable-id"))
			{
				currentElement = new OutputVariable();
			}
			else
			if (qName.equals("ordered-tlv-list"))
			{
				currentElement = new OrderedTLVList();
			}
			else
			if (qName.equals("source-variable-id"))
			{
				currentElement = new Source();
			}
			else
			if (qName.equals("page-reference"))
			{
				currentElement = new PageReference();
			}
			else
			if (qName.equals("gateway-address"))
			{
				currentElement = new GatewayAddress(atts);
			}
			else				
			if (qName.equals("submit-data"))
			{
				currentElement = new SubmitData();	
			}
			else
			if (qName.equals("anchor-reference"))
			{
				currentElement = new AnchorReference();
			}
			else
			if (qName.equals("submit-configuration"))
			{
				currentElement = new SubmitConfiguration(atts);
			}
			else
			if (qName.equals("result-info"))
			{
				currentElement = new ResultInfo();
			}
			//***********************************
			// ui-me-msg element and sub-elements
			//***********************************
			else
			if (qName.equals("uio-me-msg"))
			{
				currentElement = new UioMeMsg();
			}			
			else
			if (qName.equals("meo-ui-msg"))
			{
				currentElement = new MeoUiMsg();
			}			
			else
			if (qName.equals("usat-display-text"))
			{
				currentElement = new UsatDisplayText();
			}			
			else
			if (qName.equals("usat-select-item"))
			{
				currentElement = new UsatSelectItem();
			}			
			else
			if (qName.equals("usat-set-up-menu"))
			{
				currentElement = new UsatSetUpMenu();
			}			
			else
			if (qName.equals("usat-set-up-event-list"))
			{
				currentElement = new UsatSetUpEventList();
			}
			else
			if (qName.equals("usat-get-input"))
			{
				currentElement = new UsatGetInput();
			}				
			else
			if (qName.equals("terminal-response"))
			{
				currentElement = new TerminalResponse();
			}			
			else
			if (qName.equals("envelope-menu-selection"))
			{
				currentElement = new EnvMenuSelection();
			}			
			else
			if (qName.equals("envelope-event-download"))
			{
				currentElement = new EnvEventDownload();
			}			
			else
			if (qName.equals("command-details"))
			{
				currentElement = new CommandDetails(atts);
			}
			else
			if (qName.equals("device-identities"))
			{
				currentElement = new DeviceIdentities(atts);
			}			
			else
			if (qName.equals("text-string"))
			{
				currentElement = new TextString(atts);
			}			
			else
			if (qName.equals("alpha-identifier"))
			{
				currentElement = new AlphaIdentifier();
			}			
			else
			if (qName.equals("item-data-object"))
			{
				currentElement = new ItemDataObject(atts);
			}			
			else
			if (qName.equals("icon-descriptor"))
			{
				currentElement = new IconDescriptor(atts);
			}			
			else
			if (qName.equals("event-list"))
			{
				currentElement = new EventList();
			}			
			else
			if (qName.equals("event-type"))
			{
				currentElement = new EventType(atts);
			}			
			else
			if (qName.equals("result"))
			{
				currentElement = new Result(atts);
			}			
			else
			if (qName.equals("TR-additional-information"))
			{
				currentElement = new TRAdditionalInfo();
			}			
			if (qName.equals("item-identifier"))
			{
				currentElement = new ItemIdentifier();
			}			
			else
			if (qName.equals("help-request"))
			{
				currentElement = new HelpRequest();
			}			
			else
			if (qName.equals("event-download-additional-information"))
			{
				currentElement = new EventDownloadAdditionalInfo();
			}
			else
			if (qName.equals("response-length"))
			{
				currentElement = new ResponseLength();
			}

         parent.add(currentElement);
		} 
   }

   public void endElement(String namespaceURI,
                       String localName,
                       String qName)
                throws SAXException
   {
	
	 //trim the buffer before putting it into the element
     String s1=(buffer.toString()).trim();

	 StringBuffer IndentString = new StringBuffer();
	 for (int i=1; i<= indentLevel; i++)
		IndentString.append("   ");

	 if (s1.length() > 0) {
		System.out.println(IndentString + "ending the element " + qName +" whose value is "+s1);

		currentElement.setValue(s1);
		buffer.setLength(0);
     }
	 else
		System.out.println(IndentString + "ending the element " + qName);

     indentLevel--;


     currentElement = (Element) parentStack.pop();
   }

	public void endDocument()
                 throws SAXException {
      //generate the output
      ByteArrayOutputStream output= new ByteArrayOutputStream();
      rootElement.generate(output);
	}


   public void characters(char[] characters, int start, int length) {
      buffer.append(characters, start, length);
   }

}
