package UI_Converter.bytecode;

import UI_Converter.util.Element;
import UI_Converter.common.Tag;

/**
 * An AnchorReference object stores the content of the anchor-reference XML
 * element and generates the corresponding Anchor Reference UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class AnchorReference extends Element {
   /**
    * Create a new PageReference object.
    */
   public AnchorReference() {
      setTagValue(Tag.ANCHOR_REFERENCE);
   }

   /**
    * Set the value of the Anchor Reference.
    *
    * @param value Value of the Anchor Reference.
    *
    * @throws IllegalArgumentException if the specified value is not valid.
    */
   public void setValue(String value)
                 throws IllegalArgumentException {
      int index = value.indexOf('#');
      if (index != -1) {
         index = value.indexOf('#', index + 1);
         if (index != -1) {
            throw new IllegalArgumentException("Invalid anchor reference: " + value);
         }
      }
      super.setValue(value);
   }
}