package UI_Converter.bytecode;

/**
 * The Attribute class contains constants related to the UI-TLV attributes.
 *
 * @author Philippe Sidler (SchlumbergerSema, sidler@montrouge.sema.slb.com)
 *
 * @see 3GPP TS 31.113 V5.0.0
 */

/**
IMPORTANT : the numbering of the attributes starts with 0
**/
public class Attribute {
   /**
    * Page attribute related constants.
    */
   public static class Page {
      /**
       * Index of the Keep All attribute.
       */
      public static final int KEEP_ALL = 1;

      /**
       * Index of the Dynamic/Static attribute.
       */
      public static final int DYNAMIC = 2;

      /**
       * Index of the DCS attribute.
       */
      public static final int UCS2 = 3;
   }

   /**
    * Navigation Unit attribute related constants.
    */
   public static class NavigationUnit {
      /**
       * Index of the Reset Var attribute.
       */
      public static final int RESET_VAR = 1;

      /**
       * Index of the Do Not Historize attribute.
       */
      public static final int DO_NOT_HISTORIZE = 2;

      /**
       * Index of the Chain Next NU attribute.
       */
      public static final int CHAIN_NEXT_NU = 3;

      /**
       * Index of the Do Not Inherit Terminal Response Handler Configuration
       * attribute.
       */
      public static final int DO_NOT_INHERIT_TERMINAL_RESPONSE_HANDLER_CONFIGURATION = 4;
   }

    /**
    * Terminal Response Handler Modifier attribute related constants.
    */
   public static class TRHM {
      /**
       * Modification type attribute.
       */
      public static final int REPLACE	= 0;
      public static final int ADD		= 1;
      public static final int RESTORE	= 2;
      public static final int REMOVE	= 3;

      /**
       * Offset of the Modification type attribute.
       */
      public static final int MODIFICATION_TYPE_OFFSET = 0;
   }

   /**
    * Action attribute related constants.
    */
   public static class Action {
      /**
       * Index of the execution  attribute.
       */
      public static final int EXECUTE = 1;
   }

   /**
    * Inline Value attribute related constants.
    */
   public static class InlineValue {
      /**
       * Offset of the Value type information in the attribute byte.
       */
      public static final int VALUE_TYPE_INFORMATION_OFFSET = 0;
   }

   /**
    * Submit Configuration attribute related constants.
    */
   public static class SubmitConfiguration {
      /**
       * Index of the Send Referer attribute.
       */
      public static final int SEND_REFERER = 1;

      /**
       * Index of the Processing Behaviour attribute.
       */
      public static final int NO_WAIT_STATE = 2;
   }

   /**
    * Gateway address attribute related constants.
    */
   public static class GatewayAddress {
      /**
       * Presence of additional information attribute.
       */
      public static final int SEND_ADDITIONAL_INFORMATION = 1;

    }

   /**
    * Go Back attribute related constants.
    */
   public static class GoBack {
      /**
       * Index of the Restart Current NU attribute.
       */
      public static final int RESTART_CURRENT_NU = 1;
   }

   /**
    * Exit attribute related constants.
    */
   public static class Exit {
      /**
       * Index of the Terminate Session attribute.
       */
      public static final int TERMINATE_SESSION = 1;
   }

   /**
    * Execute USAT Command attribute related constants.
    */
   public static class ExecuteUSATCommand {
      /**
       * Index of the Result Optimization Required attribute.
       */
      public static final int RESULT_OPTIMIZATION_REQUIRED = 1;

      /**
       * Offset of the Behaviour attribute.
       */
      public static final int BEHAVIOUR_OFFSET = 1;

      public static final int CONTINUE_ANY_GR			= 0x01;
      public static final int STOP_GRVAR_NOT_PRESENT	= 0x00;
      public static final int STOP_GRVAR_PRESENT		= 0x02;
      public static final int INVOKETRH_ANY_GR			= 0x03;

      /**
       * Index of the Output Variable attribute.
       */
      public static final int OUTPUT_VARIABLE = 4;
   }

   /**
    * Execute Native Command attribute related constants.
    */
   public static class ExecuteNativeCommand {
      /**
       * Index of the Exit attribute.
       */
      public static final int EXIT = 1;
   }

   /**
    * Display Text attribute related constants.
    */
   public static class DisplayText {
      /**
       * Index of the User Interaction attribute.
       */
      public static final int CLEAR_AFTER_DELAY = 1;
   }

   /**
    * Get Input attribute related constants.
    */
   public static class GetInput {
      /**
       * Index of the UCS2 attribute.
       */
      public static final int UCS2 = 1;

      /**
       * Index of the Alphabet set attribute.
       */
      public static final int ALPHABET_SET = 2;

      /**
       * Offset of the Minimum length attribute.
       */
      public static final int MINIMUM_LENGTH_OFFSET = 2;
      public static final int MINIMUM_LENGTH_MASK   = 0x0F;
   }

}