package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import org.xml.sax.Attributes;
import UI_Converter.common.Tag;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;

/**
 * A GetTLVValue object stores the content of the get-TLV-value XML element and
 * generates the corresponding Get TLV Value UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class GetTLVValue extends Element {
   /**
    * Tag value to search for.
    */
   private int tag;

   /**
    * Create a new GetTLVValue object.
    *
    * @param atts Attributes of the get-TLV-value XML element.
    */
   public GetTLVValue(Attributes atts) {
      setTagValue(Tag.GET_TLV_VALUE);
      tag = Conversion.byteToInt(Conversion.hexaStringToByte(atts.getValue("tag")));
   }

   /**
    * Generate the byte code for the Get TLV Value.
    *
    * @param buffer
    *
    * @return the number of generated bytes.
    */
   public int generate(ByteArrayOutputStream buffer) {
      // Generate the Variable Identifier List TLV
      int length = getChild(1).generate(buffer);
      // Generate the tag value
      buffer.write(tag);
      length ++;
      // Generate the Variable Identifier
      length += getChild(0).generate(buffer);
      // Generate the length and tag
      length += generateBERTLVLength(length, buffer);
      length += generateTag(buffer);
      return length;
   }
}
