package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import org.xml.sax.Attributes;
import UI_Converter.bytecode.Attribute;
import UI_Converter.bytecode.Coding;
import UI_Converter.common.Tag;
import UI_Converter.util.Element;

/**
 * An InlineValue object stores the content of the inline-value XML element and
 * generates the corresponding Inline Value UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class InlineValue extends Element {

   /**
    * Create a new InlineValue object.
    *
    * @param atts Attributes of the inline-value XML element.
    *
    * @throw IllegalArgumentException if an XML attribute value is invalid.
    */
   public InlineValue(Attributes atts)
               throws IllegalArgumentException {
      this(Tag.INLINE_VALUE, atts);
   }

   /**
    * Create a new InlineValue object with the specified tag.
    *
    * @param tag UI-TLV tag.
    * @param atts Attributes of the inline-value XML element.
    *
    * @throw IllegalArgumentException if an XML attribute value is invalid.
    */
   protected InlineValue(byte tag, Attributes atts)
                  throws IllegalArgumentException {
      setTagValue(tag);

      String codingValue = atts.getValue("coding");
	  if (codingValue!=null)
	  {
		if (codingValue.equals("unknown")) {
         setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.UNKNOWN);
		}
		else if (codingValue.equals("sms")) {
         setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.SMS);
		}
		else if (codingValue.equals("sms-packed")) {
         setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.SMS_PACKED);
		}
		else if (codingValue.equals("binary")) {
         setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.BINARY);
		}
		else if (codingValue.equals("ucs2")) {
         setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.UCS2);
		}
		else {
         throw new IllegalArgumentException("Invalid coding value: " + codingValue);
		}
	  }
	  else setAttribute(Attribute.InlineValue.VALUE_TYPE_INFORMATION_OFFSET, Coding.UNKNOWN);
   }

	/**
    * Generate the byte code.
    *
    * @param buffer
    *
    * @return the number of generated bytes.
    */
   public int generate(ByteArrayOutputStream buffer) {
      // Generate the Length-Value or Variable Substitution Indicator-Variable
      // ID pairs.
      int length = 0;
      for (int i = getChildrenCount() - 1; i >= 0; i --) {
         length += ((Pair) getChild(i)).generatePair(buffer);
      }
      // Generate the length and tag
      length += generateBERTLVLength(length, buffer);
      length += generateTag(buffer);
      return length;
   }
}