package UI_Converter.bytecode;

import org.xml.sax.Attributes;
import UI_Converter.bytecode.Attribute;
import UI_Converter.common.Tag;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;

/**
 * An Input object stores the content of the input XML element and generates the
 * corresponding Get Input UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class Input extends Element {
   /**
    * Create a new Input object.
    *
    * @param atts Attributes of the input XML element.
    *
    * @throws IllegalArgumentException if an attribute is not valid.
    */
   public Input(Attributes atts)
         throws IllegalArgumentException {
      setTagValue(Tag.GET_INPUT);

      String inputTypeValue = atts.getValue("charset");
	  if (inputTypeValue != null){
		if (inputTypeValue.equals("sms")) {
		  // SMS is the default value. Do not change it.
		}
		else if (inputTypeValue.equals("ucs2")) {
		  setAttribute(Attribute.GetInput.UCS2);
		}
		else {
		  throw new IllegalArgumentException("Invalid input type value: " + inputTypeValue);
		}
	  }

      String formatValue = atts.getValue("format");
	  if (formatValue != null){
		if (formatValue.equals("digit")) {
		  // Digits is the default value. Do not change it.
		}
		else if (formatValue.equals("alpha")) {
		  setAttribute(Attribute.GetInput.ALPHABET_SET);
		}
		else {
		  throw new IllegalArgumentException("Invalid format value: " + formatValue);
		}
	  }

	  if (atts.getValue("minlength") != null){
        int minValue = Conversion.stringToInt(atts.getValue("minlength"));
		if (minValue >= 0 && minValue < 16) {
		  setAttribute(Attribute.GetInput.MINIMUM_LENGTH_OFFSET, minValue);
		}
		else {
		  throw new IllegalArgumentException("Invalid min value: " + minValue);
		}
	  }
   }
}