package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import org.xml.sax.Attributes;
import UI_Converter.bytecode.Coding;
import UI_Converter.bytecode.Variable;
import UI_Converter.util.Conversion;

public class Reference extends Pair {
   /**
    * Variable identifier.
    */
   private byte id;

   /**
    * Variable type.
    */
   private int coding;

   /**
    * Variable area type.
    */
   private int area;

   /**
    * Create a new Reference object.
    *
    * @param atts Attributes of the XML element.
    *
    * @throws IllegalArgumentException if an attribute is not valid.
    */
   public Reference(Attributes atts)
               throws IllegalArgumentException {

      String areaValue = atts.getValue("variable-type");

      if (areaValue.equals("env")) {
         area = Variable.FIRST_ENVIRONMENT_VARIABLE;
      }
      else if (areaValue.equals("perm")) {
         area = Variable.FIRST_PERMANENT_VARIABLE;
      }
      else if (areaValue.equals("temp")) {
         area = Variable.FIRST_TEMPORARY_VARIABLE;
      }
      else if (areaValue.equals("page-string")) {
         area = Variable.FIRST_STRING_ELEMENT;
      }
      else {
         throw new IllegalArgumentException("Invalid variable area type: " + areaValue);
      }

	  String codingValue = atts.getValue("value-type");

      if (codingValue.equals("unknown")) {
         coding = Coding.UNKNOWN;
      }
      else if (codingValue.equals("sms")) {
         coding = Coding.SMS;
      }
      else if (codingValue.equals("sms-packed")) {
         coding = Coding.SMS_PACKED;
      }
      else if (codingValue.equals("binary")) {
         coding = Coding.BINARY;
      }
      else if (codingValue.equals("ucs2")) {
         coding = Coding.UCS2;
      }
      else {
         throw new IllegalArgumentException("Invalid coding value: " + codingValue);
      }
   }
   
   
   /**
    * Generate the byte code of the variable identifier.
    *
    * @param buffer Buffer in which the byte code is generated.
    *
    * @return the number of generated bytes.
    */
   public int generate(ByteArrayOutputStream buffer) {
      buffer.write(Conversion.byteToInt(id));
      return 1;
   }

   /**
    * Generate the byte code of the Variable Substitution Indicator-Variable
    * Identifier pair.
    *
    * @param buffer Buffer in which the byte code is generated.
    *
    * @return the number of generated bytes.
    */
   public int generatePair(ByteArrayOutputStream buffer) {
      buffer.write(Conversion.byteToInt(id) | area);
      buffer.write(Variable.SUBSTITUTION_INDICATOR | coding);
      return 2;
   }

   /**
    * Return the identifier of the variable specified by its name.
    *
    * @param variableName Variable name.
    *
    * @throws IllegalArgumentException if the variable name is not valid.
    */
   protected byte getVariableIdentifier(String variableName)
	   throws IllegalArgumentException{

	   return (byte) (Conversion.stringToInt(variableName) | area);
   }

   /**
    * Set the value of the variable identifier XML element.
    *
    * @param value Value of the element.
    *
    * @throws IllegalArgumentException if the value is not valid.
    */
   public void setValue(String value)
                 throws IllegalArgumentException {
      id = getVariableIdentifier(value);
   }

	protected byte getVarId()
	{
		return id;
	}

}
