package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import org.xml.sax.Attributes;
import UI_Converter.common.Tag;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;

/**
 * A SimpleTLVIndicator object stores the content of the simple-tlv-indicator XML
 * element and generates the corresponding Simple TLV Indicator UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class SimpleTLVIndicator extends Element {
   /**
    * Result tag value.
    */
   private byte tag;

   /**
    * Create a new SimpleTLVIndicator object.
    *
    * @param atts Attributes of the USAT-command-tlv XML element.
    *
    * @throws IllegalArgumentException if an attribute value is not valid.
    */
   public SimpleTLVIndicator(Attributes atts) {
      setTagValue(Tag.SIMPLE_TLV_INDICATOR);
      tag = Conversion.hexaStringToByte(atts.getValue("id"));
   }

   /**
    * Generate the byte code.
    *
    * @param buffer
    *
    * @return the number of generated bytes.
    */
   public int generate(ByteArrayOutputStream buffer) {
      // Generate the Length-Value or Variable Substitution Indicator-Variable
      // ID pairs.
      int length = 0;
      for (int i = getChildrenCount() - 1; i >= 0; i --) {
         length += ((Pair) getChild(i)).generatePair(buffer);
      }
      // Generate the Result Tag
      buffer.write(tag);
      length ++;
      // Generate the length and tag
      length += generateBERTLVLength(length, buffer);
      length += generateTag(buffer);
      return length;
   }
}
