package UI_Converter.bytecode;

import org.xml.sax.Attributes;
import UI_Converter.bytecode.Attribute;
import UI_Converter.common.Tag;
import UI_Converter.util.Conversion;
import UI_Converter.util.Element;

/**
 * A SubmitConfiguration object stores the content of the submit-configuration
 * XML element and generates the corresponding Submit Configuration UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class SubmitConfiguration extends Element {
   /**
    * Create a new SubmitConfiguration object.
    *
    * @param atts Attributes of the submit-configuration XML element.
    */
   public SubmitConfiguration(Attributes atts) {
      setTagValue(Tag.SUBMIT_CONFIGURATION);
      boolean sendReferer = Conversion.yesNoStringToBoolean(atts.getValue("send-ref"));
      if (sendReferer) {
         setAttribute(Attribute.SubmitConfiguration.SEND_REFERER);
      }

      //handle the attribute "Processing Behavior"
	  if (atts.getValue("waitstate")!=null)
		if (atts.getValue("waitstate").equals("next-uicmd")) setAttribute(Attribute.SubmitConfiguration.NO_WAIT_STATE);
   }
}