package UI_Converter.bytecode;

import java.io.ByteArrayOutputStream;
import UI_Converter.common.Tag;
import UI_Converter.util.Element;

/**
 * A SubmitData object stores the content of the submit-data XML element and
 * generates the corresponding Submit Data UI-TLV.
 *
 *
 * @see 3GPP TS XX.YYY V0.0.1
 * @see 3GPP TS 31.113 V5.0.0
 */
public class SubmitData extends Element {
   /**
    * Create a new SubmitData object.
    */
   public SubmitData() {
      setTagValue(Tag.SUBMIT_DATA);
   }

	/**
    * Generate the byte code.
    *
    * @param buffer
    *
    * @return the number of generated bytes.
    */
   public int generate(ByteArrayOutputStream buffer) {
      // Generate the Length-Value or Variable Substitution Indicator-Variable
      // ID pairs.
      int length = 0;
      for (int i = getChildrenCount() - 1; i >= 0; i --) {
         length += ((Pair) getChild(i)).generatePair(buffer);
      }
      // Generate the length and tag
      length += generateBERTLVLength(length, buffer);
      length += generateTag(buffer);
      return length;
   }
}