package UI_Converter.common;

/**
 * The Tag class contains constants related to the UI-TLV tags.
 *
 * @author Philippe Sidler (SchlumbergerSema, sidler@montrouge.sema.slb.com)
 *
 * @see 3GPP TS 31.113 V5.0.0
 */
public class Tag {

	// tags for byte code
   public static final byte PAGE                               = 0x01;
   public static final byte PAGE_IDENTIFICATION                = 0x02;
   public static final byte PAGE_UNLOCK_CODE                   = 0x03;
   public static final byte ONE_TIME_PASSWORD                  = 0x04;
   public static final byte KEEP_ALIVE_LIST                    = 0x05;
   public static final byte SERVICE_ID                         = 0x06;
   public static final byte STRING_POOL                        = 0x07;
   public static final byte TERMINAL_RESPONSE_HANDLER_MODIFIER = 0x08;
   public static final byte ACTION                             = 0x09;
   public static final byte NAVIGATION_UNIT                    = 0x0A;
   public static final byte ANCHOR                             = 0x0B;
   public static final byte ANCHOR_REFERENCE                   = 0x0C;
   public static final byte VARIABLE_IDENTIFIER_LIST           = 0x0D;
   public static final byte INLINE_VALUE                       = 0x0E;
   public static final byte INLINE_VALUE2                      = 0x0F;
   public static final byte INPUT_LIST                         = 0x10;
   public static final byte ORDERED_TLV_LIST                   = 0x11;
   public static final byte PAGE_REFERENCE                     = 0x12;
   public static final byte SUBMIT_CONFIGURATION               = 0x13;
   public static final byte SUBMIT_DATA                        = 0x14;
   public static final byte GATEWAY_ADDRESS                    = 0x15;
   public static final byte SUBMIT                             = 0x16;

   public static final byte SET_VARIABLE                       = 0x40;
   public static final byte ASSIGN_AND_BRANCH                  = 0x41;
   public static final byte EXTRACT                            = 0x42;
   public static final byte GO_BACK                            = 0x43;
   public static final byte BRANCH_ON_VARIABLE_VALUE           = 0x44;
   public static final byte EXIT                               = 0x45;
   public static final byte EXECUTE_USAT_COMMAND               = 0x46;
   public static final byte EXECUTE_NATIVE_COMMAND             = 0x47;
   public static final byte GET_LENGTH                         = 0x48;
   public static final byte GET_TLV_VALUE                      = 0x49;
   public static final byte DISPLAY_TEXT                       = 0x4A;
   public static final byte GET_INPUT                          = 0x4B;

   public static final byte SIMPLE_TLV_INDICATOR               = 0x00;
      
	// tags for transport level
	public static final byte UIO_MSG_PULL						= (byte)0xC0;	
	public static final byte UIO_MSG_ADMIN						= (byte)0xC1;
	public static final byte GWO_MSG_PULL						= (byte)0xC2;
	public static final byte GWO_MSG_PUSH						= (byte)0xC3;
	public static final byte GWO_MSG_ADMIN						= (byte)0xC4;

	public static final byte REQUEST_ID							= 0x40;
	public static final byte ADDITIONAL_INFO					= 0x41;
	public static final byte TAR								= 0x42;	
	public static final byte ICON_ID							= 0x43;	
	public static final byte PARAMETER_LIST						= 0x44;	
	public static final byte ADMIN_RESULT						= 0x45;	
	public static final byte PLUGIN_INSTALLATION_DATA			= 0x46;	

	public static final byte ADMIN_REQUEST						= 0x60;
	public static final byte ADMIN_RESPONSE						= 0x61;
	public static final byte INSTALL_PAGE						= 0x62;	
	public static final byte REMOVE_PAGE						= 0x63;	
	public static final byte CONFIGURE_MENU_TITLE				= 0x64;	
	public static final byte CONFIGURE_MENU_TEXT_FOR_PAGE		= 0x65;	
	public static final byte REMOVE_MENU_TEXT_FOR_PAGE			= 0x66;	
	public static final byte INSTALL_EVENT						= 0x67;	
	public static final byte REMOVE_EVENT						= 0x68;
	public static final byte MODIFY_ENV_VAR						= 0x69;
	public static final byte MODIFY_WAIT_STATE_MSG				= 0x6A;	
	public static final byte MODIFY_CONFIG_DATA					= 0x6B;	
	public static final byte MODIFY_BEARER_DATA					= 0x6C;	
	public static final byte INSTALL_PLUGIN						= 0x6D;	
	public static final byte REMOVE_PLUGIN						= 0x6E;	


	public static final byte UI_VERSION							= 0x01;
	public static final byte UI_PROFILE							= 0x02;
	public static final byte HASH_URL							= 0x08;
	public static final byte RECEPTION_BUFFER_SIZE				= 0x09;
	public static final byte TRANSMISSION_BUFFER_SIZE			= 0x0B;

	// tags for ui-me-msg
	public static final byte MENU_SELECTION						= (byte)0xD3;
	public static final byte EVENT_DOWNLOAD						= (byte)0xD6;
	public static final byte PROACTIVE_SIM_SESSION				= (byte)0xD0;
	public static final byte COMMAND_DETAILS					= (byte)0x01;
	public static final byte DEVICE_IDENTITIES					= (byte)0x02;
	public static final byte RESULT								= (byte)0x03;
	public static final byte ALPHA_IDENTIFIER					= (byte)0x05;
	public static final byte TEXT_STRING						= (byte)0x0D;
	public static final byte ITEM								= (byte)0x0F;
	public static final byte ITEM_IDENTIFIER					= (byte)0x10;
	public static final byte HELP_REQUEST						= (byte)0x15;
	public static final byte ICON_DESCRIPTOR					= (byte)0x1E;
	public static final byte EVENT_LIST							= (byte)0x19;
	public static final byte RESPONSE_LENGTH					= (byte)0x11;

}
