package UI_Converter.securedmsg;

import org.xml.sax.Attributes;
import UI_Converter.common.Tag;
import UI_Converter.util.Element;
import java.io.ByteArrayOutputStream;
import UI_Converter.util.Conversion;
import UI_Converter.securedmsg.SecurityConstants;
import UI_Converter.util.FileManager;

import java.security.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;
import com.sun.crypto.provider.SunJCE;


public class SecHeaders extends Element
{

	private byte[] spi;
	private byte kic;
	private byte kid;
	private byte[] tar;
	private byte[] cntr;
	private byte pcntr = 0;
	private byte[] cc;

	public SecHeaders()
	{
		tar = new byte[SecurityConstants.TAR_LENGTH];
		for (int i=0;i<SecurityConstants.TAR_LENGTH;i++) tar[i]=(byte)0x00;
		cntr = new byte[SecurityConstants.CNTR_LENGTH];
		for (int i=0;i<SecurityConstants.CNTR_LENGTH;i++) cntr[i]=(byte)0x00;
		spi= new byte[SecurityConstants.SPI_LENGTH];
		for (int i=0;i<SecurityConstants.SPI_LENGTH;i++) spi[i]=(byte)0x00;
	}

	protected void setSpi(byte[] b)
	{
		// attention : the length of b may not be the same as spi.
		// in this case, the "00" must be AT THE BEGINNING of spi.
		for (int j=0; j<b.length;j++) spi[SecurityConstants.SPI_LENGTH-b.length+j]=b[j];	
	}

	
	protected void setKic(byte b)
	{
		kic=b;
	}

	protected void setKid(byte b)
	{
		kid=b;
	}

	protected void setTar(byte[] tarValue)
	{
		// attention : the length of tarValue may not be the same as tar.
		// in this case, the "00" must be AT THE BEGINNING of tar.
		for (int j=0; j<tarValue.length;j++) tar[SecurityConstants.TAR_LENGTH-tarValue.length+j]=tarValue[j];
	}

	protected void setCntr(byte[] cntrValue)
	{
		// attention : the length of cntrValue may not be the same as cntr.
		// in this case, the "00" must be AT THE BEGINNING of cntr
		for (int j=0; j<cntrValue.length;j++) cntr[SecurityConstants.CNTR_LENGTH-cntrValue.length+j]=cntrValue[j];
			
	}


	// This method converts a double TDES key into a triple TDES key
	// This first block of the input key is copied in the last block of the output key
	private byte[] Convert2KeysTo3Keys(byte[] InputKey){

		byte[] OutputKey = new byte[SecurityConstants.TDES_3KEY_LENGTH];
		int i;

		for(i=0; i< SecurityConstants.TDES_2KEY_LENGTH; i++){
			OutputKey[i] = InputKey[i];
		}

		for(i=0; i< (SecurityConstants.TDES_3KEY_LENGTH - SecurityConstants.TDES_2KEY_LENGTH); i++){
			OutputKey[SecurityConstants.TDES_2KEY_LENGTH + i] = InputKey[i];
		}

		return OutputKey;
	}


	public byte[] generate(byte[] clearSD)
		throws	javax.crypto.NoSuchPaddingException,
				javax.crypto.BadPaddingException,
				javax.crypto.IllegalBlockSizeException,
				java.security.NoSuchAlgorithmException,
				java.security.InvalidKeyException,
				java.security.InvalidAlgorithmParameterException
   {
		int length=0;
		String Signature_algo = null;
		String Ciphering_algo = null;

		byte[] KId_desKeyData = null;
		byte[] KIc_desKeyData = null;

		SecretKeySpec KId_desKey = null;
		SecretKeySpec KIc_desKey = null;

		byte[] cleartext;
		int CPL, CHL;
		int padding_length_for_MAC = 0;
		int text_length_for_ciphering = 0;
		int cleartext_length;

		int i;

		// Initialise some objects for crypto operations
		Cipher desCipher;
		SunJCE jce;
		byte[] IV = new byte[SecurityConstants.CIPHER_BLOCK_SIZE];
		IvParameterSpec IVobject = new IvParameterSpec(IV);


		// get the algorithm for signature
		// we know that for USATI the only values for RC/CC/DS are:
		// none or "CC" which is made either with DES CBC or TDES CBC with 2 keys
		if ((spi[0] & 0x02) == 0x02){

			if ((kid & 0x03) == 0x01){
				if ((kid & 0x0C) == 0x00){
					Signature_algo = "DES/CBC/NoPadding";
				}
				else
				if ((kid & 0x0C) == 0x04){
					Signature_algo = "DESede/CBC/NoPadding";
				}
				else{
					System.out.println("Error: Pb with KId value: DES CBC or TDES CBC 2 keys are mandatory");
					return null;
				}
			}
			else{
				System.out.println("Error: Pb with KId value: DES algorithm is mandatory");
				return null;
			}
		}

		// get the algorithm for ciphering
		// we know that for USATI the only values for ciphering are:
		// none, or DES CBC ot TDES CBC with 2 keys
		if ((spi[0] & 0x04) == 0x04){
			if ((kic & 0x03) == 0x01){
				if ((kic & 0x0C) == 0x00){
					Ciphering_algo = "DES/CBC/NoPadding";
				}
				else
				if ((kic & 0x0C) == 0x04){
					Ciphering_algo = "DESede/CBC/NoPadding";
				}
				else{
					System.out.println("Error: Pb with KIc value: DES CBC or TDES CBC 2 keys are mandatory");
					return null;
				}
			}
			else{
				System.out.println("Error: Pb with KIc value: DES algorithm is mandatory");
				return null;
			}		
		}


		// get the signature key value
		if (Signature_algo != null){
			KId_desKeyData = FileManager.GetSignatureKeyFromKeyFile((byte)((kid & 0xF0) >> 4));
		
			if (	( (KId_desKeyData.length == SecurityConstants.DES_KEY_LENGTH)	&& (Signature_algo.compareTo("DESede/CBC/NoPadding")	== 0) )
				||	( (KId_desKeyData.length == SecurityConstants.TDES_2KEY_LENGTH)	&& (Signature_algo.compareTo("DES/CBC/NoPadding")		== 0	) ) ){
				System.out.println("Error: Signature key coding inconsistent with the key size in TransportKey file");
				return null;
			}
			if (Signature_algo.equals("DESede/CBC/NoPadding")){
				byte[] KId_desKeyData_3_keys = Convert2KeysTo3Keys(KId_desKeyData);
				KId_desKey = new SecretKeySpec(KId_desKeyData_3_keys, "DESede");
			}
			else{
				KId_desKey = new SecretKeySpec(KId_desKeyData, "DES");
			}
		}


		// get the signature key value
		if (Ciphering_algo != null){
			KIc_desKeyData = FileManager.GetCipherKeyFromKeyFile((byte)((kic & 0xF0) >> 4));
		
			if (	( (KIc_desKeyData.length == SecurityConstants.DES_KEY_LENGTH)	&& (Ciphering_algo.compareTo("DESede/CBC/NoPadding")	== 0) )
				||	( (KIc_desKeyData.length == SecurityConstants.TDES_2KEY_LENGTH)	&& (Ciphering_algo.compareTo("DES/CBC/NoPadding")		== 0	) ) ){
				System.out.println("Error: Ciphering key coding inconsistent with the key size in TransportKey file");
				return null;
			}
			if (Ciphering_algo.equals("DESede/CBC/NoPadding")){
				byte[] KIc_desKeyData_3_keys = Convert2KeysTo3Keys(KIc_desKeyData);
				KIc_desKey = new SecretKeySpec(KIc_desKeyData_3_keys, "DESede");
			}
			else
				KIc_desKey = new SecretKeySpec(KIc_desKeyData, "DES");
		}


		// Create the 03.48 message without MAC. cleartext[] byte array includes:
		// CPL, CHL, SPI, KIc, KId, TAR, CNTR, PCNTR and SD
		CHL =		SecurityConstants.SPI_LENGTH
				+	SecurityConstants.KIC_LENGTH
				+	SecurityConstants.KID_LENGTH
				+	SecurityConstants.TAR_LENGTH
				+	SecurityConstants.CNTR_LENGTH
				+	SecurityConstants.PCNTR_LENGTH;

		cleartext_length =		SecurityConstants.CPL_LENGTH
							+	SecurityConstants.CHL_LENGTH
							+	CHL
							+	clearSD.length;

		// calculate pcntr
		if (Ciphering_algo != null){
			text_length_for_ciphering =		cleartext_length
										-	SecurityConstants.CPL_LENGTH
										-	SecurityConstants.CHL_LENGTH
										-	SecurityConstants.SPI_LENGTH
										-	SecurityConstants.KIC_LENGTH
										-	SecurityConstants.KID_LENGTH
										-	SecurityConstants.TAR_LENGTH;

			if (Signature_algo != null){
				text_length_for_ciphering += SecurityConstants.CC_LENGTH;
			}

			pcntr = (byte)(SecurityConstants.CIPHER_BLOCK_SIZE - (text_length_for_ciphering % SecurityConstants.CIPHER_BLOCK_SIZE));
		}

		// calculate the number of padding bytes for signature, CPL and CHL
		if (Signature_algo != null){
			padding_length_for_MAC = SecurityConstants.CIPHER_BLOCK_SIZE - (cleartext_length % SecurityConstants.CIPHER_BLOCK_SIZE);
			
			CHL += SecurityConstants.CC_LENGTH;
			CPL = SecurityConstants.CHL_LENGTH + CHL + clearSD.length + padding_length_for_MAC;

		}
		else{
			CPL = SecurityConstants.CHL_LENGTH + CHL + clearSD.length + pcntr;

		}

		// Create cleartext[] array
		cleartext = new byte[cleartext_length + SecurityConstants.CC_LENGTH + SecurityConstants.CIPHER_BLOCK_SIZE ];

		// append CPL to cleartext[]
		if ( CPL < 256 )
		{
			cleartext[SecurityConstants.CPL_INDEX] = (byte)0x00;
			cleartext[SecurityConstants.CPL_INDEX + 1] = (byte)(CPL);
		}
		else
		{
			cleartext[SecurityConstants.CPL_INDEX] = (byte)( CPL >> 256 );
			cleartext[SecurityConstants.CPL_INDEX + 1] = (byte)(CPL);
		}

		// appendCHL to cleartext[]
		cleartext[SecurityConstants.CHL_INDEX] = (byte)CHL;

		// append SPI to cleartext[]
		for (i=0; i<spi.length;i++)
			cleartext[SecurityConstants.SPI_INDEX + i] = spi[i];

		// append KIc to cleartext[]
		cleartext[SecurityConstants.KIC_INDEX] = kic;

		// append KId to cleartext[]
		cleartext[SecurityConstants.KID_INDEX] = kid;

		// append the TAR to cleartext[]
		for (i=0; i<tar.length;i++)
			cleartext[SecurityConstants.TAR_INDEX + i] = tar[i];

		// append the CNTR to cleartext[]
		for (i=0; i<cntr.length;i++)
			cleartext[SecurityConstants.CNTR_INDEX + i] = cntr[i];

		// append the PCNTR to cleartext[]
		cleartext[SecurityConstants.PCNTR_INDEX] = pcntr;

		// append secured data (skip the MAC)
		for (i=0; i<clearSD.length ; i++)
			cleartext[SecurityConstants.CC_INDEX + i] = clearSD[i];

		// Initialise cipher object if signature or encryption are needed
		if ( (Signature_algo != null) || (Ciphering_algo != null) ){

			// Add SunJCE to the list of providers
            jce = new SunJCE();
            Security.addProvider(jce);
		}

		// if MAC is required, then insert the MAC in cleartext[]
		if (Signature_algo != null){

			// Create cipher object with the rigth algorithm
			desCipher = Cipher.getInstance(Signature_algo);

			// Initialize the cipher for encryption
			desCipher.init(Cipher.ENCRYPT_MODE, KId_desKey, IVobject);

			// Encrypt the cleartext
			byte[] ciphertext = desCipher.doFinal(cleartext, SecurityConstants.CPL_INDEX, (cleartext_length + padding_length_for_MAC));

			// Get the MAC value from ciphertext[] and insert it in cleartext[] 
			for (i = 0; i<= (SecurityConstants.CIPHER_BLOCK_SIZE - 1) ; i++)
				cleartext[SecurityConstants.CC_INDEX + i] = ciphertext[ciphertext.length - SecurityConstants.CIPHER_BLOCK_SIZE + i];

			cleartext_length += SecurityConstants.CC_LENGTH;

			// Append the secured data after the MAC
			for (i=0; i<clearSD.length ; i++)
				cleartext[SecurityConstants.SD_INDEX + i] = clearSD[i];

		}

		if (Ciphering_algo != null){

			// Create cipher object with the rigth algorithm
			desCipher = Cipher.getInstance(Ciphering_algo);

			// Initialize the cipher for encryption
			desCipher.init(Cipher.ENCRYPT_MODE, KIc_desKey, IVobject);

			// Encrypt the cleartext
			byte[] ciphertext = desCipher.doFinal(cleartext, SecurityConstants.CNTR_INDEX, text_length_for_ciphering + pcntr);

			// Append ciphered data to cleartext[] from CNTR field
			for (i=0; i<(text_length_for_ciphering + pcntr) ; i++)
				cleartext[SecurityConstants.CNTR_INDEX + i] = ciphertext[i];
		}

		// return the complete 03.48 secured message without useless bytes
		byte[] temp = new byte[CPL + SecurityConstants.CPL_LENGTH];
		for (i=0; i<CPL + SecurityConstants.CPL_LENGTH ; i++)
			temp[i] = cleartext[i];
		return temp;
   }
}






