package UI_Converter.util;

import java.util.ArrayList;
import java.io.ByteArrayOutputStream;
import UI_Converter.util.Conversion;

public class Element
{

   private ArrayList childrenElements;
   private int attributesByte;
   private int tagValue;
   protected byte[] value;

   public Element()
   {
      childrenElements = new ArrayList();
   }

   public void setValue(byte[] value) {
      this.value = value;
   }

   public void setValue(String s)
   {
      value = s.getBytes();
   }


   public void setTagValue(byte tagV)
   {
      tagValue = (int)tagV;
   }

   // add a child element to the list
   public void add(Element e)
   {
      childrenElements.add(e);
   }

   // set the attribute at the position "index"
   public void setAttribute(int index)
   {
      attributesByte = attributesByte | (1<<(index - 1));
   }

    // set the multi-bit attribute starting at offset to the specified value
   public void setAttribute(int offset, int value) {
      attributesByte |= (value << offset);
   }

   //generates the bytecode for this element
   //returns the number of bytes generated
   public int generate(ByteArrayOutputStream buffer)
   {
      int length = generateChildrenElements(buffer);
      length += generateValue(buffer);
      length += generateAttributes(buffer);
      length += generateBERTLVLength(length, buffer);
      length += generateTag(buffer);
      return length;
   }

   protected int generateAttributes(ByteArrayOutputStream buffer) {
      if (attributesByte == 0) {
         return 0;
      }
      else {
         buffer.write(attributesByte);
         return 1;
	  }
   }

   // in buffer, generate the value of "l" in the BERTLV format
   protected int generateBERTLVLength(int l, ByteArrayOutputStream buffer)
   {
      int lengthLength=0;
      if (l<128)
      {
         buffer.write(l);
         lengthLength=1;
      }
      else if (l<256)
      {
         buffer.write(l);
         buffer.write((byte)0x81);
         lengthLength=2;
      }
      else
      {
         buffer.write(l);
         buffer.write(l>>256);
         buffer.write((byte)0x82);
         lengthLength=3;
      }
      return lengthLength;
   }

    protected int generateChildrenElements(ByteArrayOutputStream buffer) {
      int length=0;
      //on ajoute le bytecode necessaire dans "buffer"
      for (int i=childrenElements.size() - 1; i>=0; i--)
      {
         length+=((Element)(childrenElements.get(i))).generate(buffer);
      }
      return length;
   }

   protected int generateValue(ByteArrayOutputStream buffer) {
      if (value == null) {
         return 0;
      }
      else {
         buffer.write(Conversion.reverseBytes(value), 0, value.length);
         return value.length;
      }
   }

   protected int generateTag(ByteArrayOutputStream buffer) {
       buffer.write(attributesByte != 0 ? (tagValue | 0x80) : tagValue);
      return 1;
   }

   protected int getChildrenCount() {
      return childrenElements.size();
   }

   public Element getChild(int index) {
      return (Element) childrenElements.get(index);
   }

   protected void removeAllChildren() {
      childrenElements.clear();
   }
}